package Bencher::Scenario::Accessors::Construction;

our $DATE = '2016-04-14'; # DATE
our $VERSION = '0.06'; # VERSION

use Bencher::ScenarioUtil::Accessors;

my $classes = \%Bencher::ScenarioUtil::Accessors::classes;

our $scenario = {
    summary => 'Benchmark object construction',
    modules => {
        'Perl::Examples::Accessors' => {version=>0.05},
    },
    participants => [
        map {
            my $spec = $classes->{$_};
            +{
                name => $spec->{generator} || $spec->{name},
                module => $_,
                code_template => "${_}->new",
            };
        } keys %$classes,
    ],
    include_result_size => 1,
};

1;
# ABSTRACT: Benchmark object construction

__END__

=pod

=encoding UTF-8

=head1 NAME

Bencher::Scenario::Accessors::Construction - Benchmark object construction

=head1 VERSION

This document describes version 0.06 of Bencher::Scenario::Accessors::Construction (from Perl distribution Bencher-Scenarios-Accessors), released on 2016-04-14.

=head1 SYNOPSIS

To run benchmark with default option:

 % bencher -m Accessors::Construction

To run module startup overhead benchmark:

 % bencher --module-startup -m Accessors::Construction

For more options (dump scenario, list/include/exclude/add participants, list/include/exclude/add datasets, etc), see L<bencher> or run C<bencher --help>.

=head1 BENCHMARKED MODULES

Version numbers shown below are the versions used when running the sample benchmark.

L<Perl::Examples::Accessors> 0.05

L<Perl::Examples::Accessors::Array> 0.05

L<Perl::Examples::Accessors::ClassAccessor> 0.05

L<Perl::Examples::Accessors::ClassAccessorArray> 0.05

L<Perl::Examples::Accessors::ClassInsideOut> 0.05

L<Perl::Examples::Accessors::ClassXSAccessor> 0.05

L<Perl::Examples::Accessors::ClassXSAccessorArray> 0.05

L<Perl::Examples::Accessors::Hash> 0.05

L<Perl::Examples::Accessors::Mo> 0.05

L<Perl::Examples::Accessors::MojoBase> 0.05

L<Perl::Examples::Accessors::Moo> 0.05

L<Perl::Examples::Accessors::Moose> 0.05

L<Perl::Examples::Accessors::Mouse> 0.05

L<Perl::Examples::Accessors::ObjectSimple> 0.05

L<Perl::Examples::Accessors::Scalar> 0.05

=head1 BENCHMARK PARTICIPANTS

=over

=item * Mouse (perl_code)

Code template:

 Perl::Examples::Accessors::Mouse->new



=item * Class::Accessor::Array (perl_code)

Code template:

 Perl::Examples::Accessors::ClassAccessorArray->new



=item * Mojo::Base (perl_code)

Code template:

 Perl::Examples::Accessors::MojoBase->new



=item * Class::XSAccessor (perl_code)

Code template:

 Perl::Examples::Accessors::ClassXSAccessor->new



=item * Moo (perl_code)

Code template:

 Perl::Examples::Accessors::Moo->new



=item * no generator (array-based) (perl_code)

Code template:

 Perl::Examples::Accessors::Array->new



=item * Mo (perl_code)

Code template:

 Perl::Examples::Accessors::Mo->new



=item * no generator (scalar-based) (perl_code)

Code template:

 Perl::Examples::Accessors::Scalar->new



=item * Moose (perl_code)

Code template:

 Perl::Examples::Accessors::Moose->new



=item * Class::XSAccessor::Array (perl_code)

Code template:

 Perl::Examples::Accessors::ClassXSAccessorArray->new



=item * no generator (hash-based) (perl_code)

Code template:

 Perl::Examples::Accessors::Hash->new



=item * Class::InsideOut (perl_code)

Code template:

 Perl::Examples::Accessors::ClassInsideOut->new



=item * Object::Simple (perl_code)

Code template:

 Perl::Examples::Accessors::ObjectSimple->new



=item * Class::Accessor (perl_code)

Code template:

 Perl::Examples::Accessors::ClassAccessor->new



=back

=head1 SAMPLE BENCHMARK RESULTS

Run on: perl: I<< v5.22.0 >>, CPU: I<< Intel(R) Core(TM) i7-4770 CPU @ 3.40GHz (4 cores) >>, OS: I<< GNU/Linux Debian version 8.0 >>, OS kernel: I<< Linux version 3.16.0-4-amd64 >>.

Benchmark with default options (C<< bencher -m Accessors::Construction >>):

 +-----------------------------+-------------+-----------+------------+-----------------+---------+---------+
 | participant                 | rate (/s)   | time (μs) | vs_slowest | result_size (b) | errors  | samples |
 +-----------------------------+-------------+-----------+------------+-----------------+---------+---------+
 | Class::InsideOut            | 3.9e+05     | 2.6       | 1          | 144             | 6.7e-09 | 20      |
 | Moose                       | 9.4e+05     | 1.1       | 2.4        | 120             | 1.7e-09 | 20      |
 | Mo                          | 1.29865e+06 | 0.770029  | 3.36094    | 120             | 0       | 20      |
 | Moo                         | 1.66899e+06 | 0.599166  | 4.31938    | 120             | 0       | 20      |
 | Class::Accessor             | 1.81e+06    | 0.552     | 4.69       | 120             | 1.9e-10 | 23      |
 | Mouse                       | 1.89e+06    | 0.53      | 4.89       | 120             | 1.8e-10 | 28      |
 | no generator (array-based)  | 2.52e+06    | 0.397     | 6.52       | 64              | 1.6e-10 | 33      |
 | no generator (hash-based)   | 3.00673e+06 | 0.332587  | 7.78149    | 120             | 0       | 20      |
 | Object::Simple              | 3.06127e+06 | 0.326661  | 7.92265    | 120             | 0       | 20      |
 | Mojo::Base                  | 3.07997e+06 | 0.324679  | 7.97103    | 120             | 0       | 20      |
 | no generator (scalar-based) | 3.35e+06    | 0.298     | 8.67       | 72              | 1e-10   | 20      |
 | Class::Accessor::Array      | 3.38e+06    | 0.296     | 8.75       | 64              | 8.2e-11 | 32      |
 | Class::XSAccessor           | 5.05885e+06 | 0.197673  | 13.0924    | 120             | 0       | 20      |
 | Class::XSAccessor::Array    | 5.26215e+06 | 0.190036  | 13.6186    | 64              | 0       | 20      |
 +-----------------------------+-------------+-----------+------------+-----------------+---------+---------+

=head1 DESCRIPTION

Packaging a benchmark script as a Bencher scenario makes it convenient to include/exclude/add participants/datasets (either via CLI or Perl code), send the result to a central repository, among others . See L<Bencher> and L<bencher> (CLI) for more details.

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Bencher-Scenarios-Accessors>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Bencher-Scenarios-Accessors>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Bencher-Scenarios-Accessors>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 SEE ALSO

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
